=begin

Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:         importKmlLineString.rb
Author:       Andreas Eisenbarth
Usage:        menu Plugins > 'Import KML LineStrings'
Description:  It imports a <LineString> list of GPS coordinates as a 
              curve. It georeferences the model and assigns color and 
              name if provided in the kml file.
Date:         26.03.2011

=end

module ImportKml
require 'sketchup.rb'

def ImportKml.setGeo(lat,lon,alt)
  model=Sketchup.active_model
  model.set_attribute("GeoReference","Latitude",lat)
  model.set_attribute("GeoReference","Longitude",lon)
  model.set_attribute("GeoReference","LocationSource","Google Earth")
  model.set_attribute("GeoReference","UsesGeoReferencing","true")
  shadowinfo=model.shadow_info 
  shadowinfo["City"] = "Unknown"
  shadowinfo["Country"] = "Unknown"
  shadowinfo["Latitude"] = lat
  shadowinfo["Longitude"] = lon 
  return [lat, lon, alt]
end

##########
def ImportKml.run
  model=Sketchup.active_model
  if model.get_attribute("GeoReference","UsesGeoReferencing") # use georeference as origin if model is georeferenced
    origin=[model.get_attribute("GeoReference","Latitude").to_f, model.get_attribute("GeoReference","Longitude").to_f, 0]
  end

  kml=UI.openpanel "Open Kml File", "c:\\", "*.kml"
  kml=IO.read(kml)
  placemarks=kml.scan(/<Placemark.*?>(.*?)<\/Placemark/m) # find placemarks

  for p in placemarks # loop through all placemarks
    p=p[0]
    name=p.scan(/name>(.*?)<\/name/m)[0][0]
    data=p.scan(/<LineString.*?coordinates>\s*(.*?)\s*<\/coordinates.*?<\/LineString/m)
    c=data[0][0].scan(/(.*?),(.*?),(.*?)(?:\s|\n)/m) # find list of coordinates
    c.uniq! # remove duplicates 

    for k in (0..c.length-1) do for l in (0..c[k].length-1) do c[k][l]=c[k][l].to_f end end # turn strings into floating numbers
    if !origin # use first coordinate as origin if origin hasn't been set before
      origin=ImportKml.setGeo(c[0][1], c[0][0], c[0][2])
    end

    pt=[] # create list of Point3d from coordinates
    for j in (0..c.length-1) do
      pt[j]=model.latlong_to_point(c[j]) # calculate model coordinates from latlong coordinates (2d)
      pt[j].z=(c[j][2]-origin[2]).m # add 3rd coordinate what latlong_to_point cannot do
    end

    # create group with curve from points
    group=model.entities.add_group
    group.entities.add_curve pt
    group.name=name

    # apply color to group
    color=p.scan(/<Style.*?<LineStyle.*?<color>\s*(.*?)\s*<\/color>.*?<\/Style/m) # try to find a style directly in the placemark
    if color.length!=0
      color=color[0][0]
    else
      style=p.scan(/styleUrl>#(.*?)<\/styleUrl/m) # if not then try to find a link to a style definition
      if style.length!=0
        color=kml.scan(/<Style\sid="#{style[0][0]}".*?<LineStyle.*?<color>\s*(.*?)\s*<\/color>.*?<\/Style/m)[0][0]
      end
    end
    if color
      group.material=eval("0x"+color.slice(-6..-1))
      model.rendering_options["EdgeColorMode"]=0 # turn colored edges style on
    end

  end #for
end #def run
end #module ImportKml

UI.menu("Plugins").add_item("Import KML LineStrings"){ImportKml.run}
